import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/field/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/money/formatter.dart';

class MoneyFieldWidget extends StatefulWidget {
  late final String? _hint;
  late final bool _isUsingPeriodInThousand;
  late final Function(double? value)? _onChanged;
  late final String? _symbol;

  MoneyFieldWidget(
      {String? hint,
      bool isUsingPeriodInThousand = true,
      Function(double? value)? onChanged,
      String? symbol,
      Key? key})
      : super(key: key) {
    _hint = hint;
    _isUsingPeriodInThousand = isUsingPeriodInThousand;
    _onChanged = onChanged;
    _symbol = symbol;
  }

  @override
  State<MoneyFieldWidget> createState() => _State();
}

class _State extends State<MoneyFieldWidget> with FieldWidgetMixin {
  final TextEditingController _controller = TextEditingController();
  late final String _decimalSeparator, _thousandSeparator;
  late final MoneyFormatter _formatter;

  @override
  Widget build(final BuildContext context) => container(
      textField: TextField(
          controller: _controller,
          decoration:
              InputDecoration(border: InputBorder.none, hintText: widget._hint),
          inputFormatters: [
            FilteringTextInputFormatter.allow(
                RegExp('[0-9$_decimalSeparator]')),
            _formatter
          ],
          keyboardType: Platform.isAndroid
              ? const TextInputType.numberWithOptions(decimal: true)
              : TextInputType.text));

  @override
  void dispose() {
    if (widget._onChanged != null) {
      _controller.removeListener(_listener);
    }
    _controller.dispose();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    if (widget._onChanged != null) {
      _controller.addListener(_listener);
    }
    _decimalSeparator = widget._isUsingPeriodInThousand ? '.' : ',';
    _thousandSeparator = widget._isUsingPeriodInThousand ? ',' : '.';
    _formatter = MoneyFormatter(
        decimalSeparator: _decimalSeparator,
        symbol: widget._symbol,
        thousandSeparator: _thousandSeparator);
  }

  void _listener() {
    String string = _controller.text
        .replaceAll(_thousandSeparator, '')
        .replaceFirst(_decimalSeparator, '.');
    if (widget._symbol != null) {
      string = string.replaceFirst(widget._symbol!, '');
    }
    widget._onChanged?.call(double.tryParse(string));
  }
}
